/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.pool;

import org.tinygroup.pool.exception.PoolException;

/*
 TODO 增加空闲时段，调用空闲操作

 */

/**
 * Created by IntelliJ IDEA. 
 * User: luoguo 
 * Date: 11-3-29 
 * Time: 下午2:06 
 * To change
 * this template use File | Settings | File Templates.
 */
public interface Pool<T> {
	int INIT_SIZE = 10;
	int MAX_SIZE = 30;
	int INCREASE_STEP = 5;
	int MAX_USE_TIMES = 10000;


	/**
	 * 初始化
	 * 
	 * @param initSize
	 *            初始大小
	 * @param maxSize
	 *            最大大小
	 * @param increaseStep
	 *            增加步长
	 * @param testBeforeBorrow
	 *            借出前进行测试
	 * @param objectProvider
	 *            对象提供类实现
	 * @throws CreateObjectException
	 */
	void initPool(int initSize, int maxSize, int increaseStep,
			boolean testBeforeBorrow, ObjectProvider<T> objectProvider);

	/**
	 * 借出对象
	 * 
	 * @return
	 * @throws PoolException
	 * @throws CreateObjectException
	 */
	T borrowObject() throws PoolException;

	/**
	 * 还回对象
	 * 
	 * @param object
	 * @throws CreateObjectException
	 */
	void returnObject(T object);

	/**
	 * 开始服务
	 */
	void start();

	/**
	 * 停止服务
	 */
	void stop();

	/**
	 * 暂停服务
	 */
	void pause();

}
