/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.pool.impl;

import org.tinygroup.linklist.LinkList;
import org.tinygroup.pool.PoolState;
import org.tinygroup.pool.exception.PoolException;

//TODO增加使用次数

/**
 * Created by IntelliJ IDEA. User: luoguo Date: 11-3-29 Time: 下午2:21 To change
 * this template use File | Settings | File Templates.
 */
public abstract class PoolLinkList<T> extends AbstractPool<T> {

	private LinkList<TimesWithObject<T>> usingList;// 使用列表
	private LinkList<TimesWithObject<T>> idleList;// 闲置列表

	public void setUsingList(LinkList<TimesWithObject<T>> usingList) {
		this.usingList = usingList;
	}

	public void setIdleList(LinkList<TimesWithObject<T>> idleList) {
		this.idleList = idleList;
	}

	protected T borrow() throws PoolException {

		// 如果已经用完，且还没到最大可用大小，则增加池大小
		if (idleList.size() == 0 && usingList.size() < getMaxSize()) {
			int increaseStep = getIncreaseStep();
			int size = getMaxSize() - usingList.size() > increaseStep ? increaseStep
					: getMaxSize() - usingList.size();
			for (int i = 0; i < size; i++) {
				T o = getObjectProvider().createObject();
				if (o != null) {
					idleList.addLast(new TimesWithObject<T>(o));
				}
			}
		}
		nextObject: if (idleList.size() > 0) {
			TimesWithObject<T> t = idleList.removeFirst();
			if (isTestBeforeBorrow()) {
				// 如果测试通过，则返回对象
				if (getObjectProvider().test(t.getObject())) {
					return getObject(t);
				} else {// 测试不通过，则释放坏了的对象再找下一个
					getObjectProvider().releaseObject(t.getObject());
					break nextObject;
				}
			}else{
				return getObject(t);
			}
		}
		throw new PoolException("Pool is empty.");
	}

	private T getObject(TimesWithObject<T> t) {
		t.setTimes(t.getTimes() + 1);
		usingList.addLast(t);
		return t.getObject();
	}

	protected void ret(T object) {
		// 如果已经是结束状态，则自动释放之
		if (getPoolState() == PoolState.STOP.ordinal()) {
			usingList.remove(new TimesWithObject<T>(object));
			getObjectProvider().releaseObject(object);
		} else {
			TimesWithObject<T> timesWithObject = usingList
					.remove(new TimesWithObject<T>(object));
			// 如果已经超过最多使用次数，则释放老对象，创建新对象
			if (timesWithObject.getTimes() == getUseTimes()) {
				getObjectProvider().releaseObject(timesWithObject.getObject());
				T o = getObjectProvider().createObject();
				if (o != null) {

					timesWithObject = new TimesWithObject<T>(o);
				}
			}
			if (timesWithObject != null) {
				idleList.addLast(timesWithObject);
			}
			if (idleList.size() > getInitSize() + getIncreaseStep()) {
				// 如果空闲数已经大于初始数+increaseStep，此时进行回收
				for (int i = 0; i < getIncreaseStep(); i++) {
					TimesWithObject<T> t = idleList.removeLast();
					getObjectProvider().releaseObject(t.getObject());
				}
			}
		}
	}

	protected void stopProvide() {
		while (idleList.size() > 0) {
			TimesWithObject<T> two = idleList.removeLast();
			getObjectProvider().releaseObject(two.getObject());
		}
	}

	public int getIdleSize() {
		return idleList.size();
	}

	public int getUsingSize() {
		return usingList.size();
	}

	@Override
	protected void addObject(TimesWithObject<T> timesWithObject) {
		idleList.addLast(timesWithObject);

	}

}
